/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
*****************************************************************************/
/*****************************************************************************
*
*  MODULE :             Interface to Metalevel
*  SCOPE :              Interface to Metalevel
*  FILE :               dictionary.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
*****************************************************************************/

#ifdef __GNUG__
#pragma interface
#endif

#ifndef __dictionary_HXX
#define __dictionary_HXX

/* IMPORTS ******************************************************************/

#ifndef __colx_HXX
#include "colx.hxx"
#endif

#ifndef __listx_HXX
#include "listx.hxx"
#endif

#ifndef dictx_HXX
#include "dictx.hxx"
#endif

#include "dictionary_tmpl.hxx"

/****************************************************************************/
     
class ADM_Server : public ObjectX {
  public :

  StringX name ;
  StringX log ;
  LocalListX<ADM_Vol> l_vol ;

  ADM_Server(char* name) ;
  ~ADM_Server() ;
  void movelog(char* dir) ;
  void print(int lev, char* head = "") ;
  void* operator new(size_t s) { 
    return ObjectX::operator new(s, (Cluster*)YOO_ADMIN_CLU, UNTYPED) ;
  }
};

class ADM_Vol : public ObjectX {
public :
  StringX name ;
  StringX file ;
  int     volId ;
  Ref<ADM_Server> server ;
  LocalListX<ADM_Clu> l_clu ;

  ADM_Vol(char* name, char* file, Ref<ADM_Server> s = NULL) ;
  ~ADM_Vol() ;
  void print(int lev, char* head = "") ;
  int  movefile(char* file) ;
  int moveserver(Ref<ADM_Server> s) ;
  void* operator new(size_t s) { 
    return ObjectX::operator new(s, (Cluster*)YOO_ADMIN_CLU, UNTYPED) ;
  }
} ;

class ADM_Clu : public ObjectX {
public :
  StringX   name ;
  Ref<Cluster> clu ;
  Ref<ADM_Vol> myVol ;
  LocalListX<ADM_Class> l_class ;

  ADM_Clu(char* name, ADM_Vol* vol) ;
  ~ADM_Clu() ;
  void print(int lev, char* head = "") ;
  void* operator new(size_t s) { 
    return ObjectX::operator new(s, (Cluster*)YOO_ADMIN_CLU, UNTYPED) ;
  }
} ;

class ADM_Class : public ObjectX {
public :
  StringX   name ;
  StringX   super ;
  int       clId ;
  Ref<ADM_Clu> myClu ;

  ADM_Class(char* name, char* super, ADM_Clu* clu, int id = -1) ;
  ~ADM_Class() ;
  void print(char* head = "") ;
  void move(ADM_Clu*) ;
  void yoodaGenStub(FILE*) ;
  void yoodaGenRef() ;
  int  supernumber() ;
  ADM_Class* superclass(int) ;
  void* operator new(size_t s) { 
    return ObjectX::operator new(s, (Cluster*)YOO_ADMIN_CLU, UNTYPED) ;
  }
} ;

class ADM_NameObj : public ObjectX {
public :
  StringX    name ;
  yoo_ObjectX obj ;
  int clId ;
  ADM_NameObj(StringX n, yoo_ObjectX o) ;
  ~ADM_NameObj() ;
  void print(char* head = "") ;
  void* operator new(size_t s) { 
    return ObjectX::operator new(s, (Cluster*)YOO_ADMIN_CLU, UNTYPED) ;
  }
} ;

class ADM_SchFile : public ObjectX {
public :
  StringX    name ;
  ADM_SchFile(StringX n) : name(n) {} ;
  void* operator new(size_t s) { 
    return ObjectX::operator new(s, (Cluster*)YOO_ADMIN_CLU, UNTYPED) ;
  }
} ;


yoo_declare1(ListX<ADM_Vol>, ListX<ObjectX>) ;
typedef DictX<ADM_Vol, StringX, yoo_index(ADM_Vol::name)> DicoVol ;
yoo_declare1(ListX<ADM_Clu>, ListX<ObjectX>) ;
typedef DictX<ADM_Clu, StringX, yoo_index(ADM_Clu::name)> DicoClu ;
yoo_declare1(ListX<ADM_Class>, ListX<ObjectX>) ;
typedef DictX<ADM_Class, StringX, yoo_index(ADM_Class::name)> DicoClass ;
yoo_declare1(ListX<ADM_NameObj>, ListX<ObjectX>) ;
typedef DictX<ADM_NameObj, StringX, yoo_index(ADM_NameObj::name)> DicoNameObj ;
yoo_declare1(ListX<ADM_SchFile>, ListX<ObjectX>) ;
typedef DictX<ADM_SchFile, StringX, yoo_index(ADM_SchFile::name)> DicoSchFile ;
yoo_declare1(ListX<ADM_Server>, ListX<ObjectX>) ;
typedef DictX<ADM_Server, StringX, yoo_index(ADM_Server::name)> DicoServer ;
     
class ADM_Admin : public ObjectX {
public :
  int lastVol ;
  Ref<ADM_Vol> defVol ;
  DicoVol  dictVol ;
  int lastClu ;
  Ref<ADM_Clu> defClu ;
  DicoClu dictClu ;
  int lastClass ;
  Ref<ADM_Server> defServ ;
  DicoClass   dictClass ;
  DicoNameObj dictName ;
  DicoSchFile dictSch ;
  DicoServer dictServ ;
  
  ADM_Admin() ;

  void* operator new(size_t s) { 
    return ObjectX::operator new(s, (Cluster*)YOO_ADMIN_CLU, UNTYPED) ;
  }

  Ref<Cluster> getCluster(char* name) {
    ADM_Clu* cluster = this->dictClu.getFirst(name) ;
    if (cluster == NULL) {
      return NULL ;
    } else {
      return cluster->clu ;
    }
  }
} ;

extern Ref<ADM_Admin> YoodaDictionary ;

#endif
